@@ -1,6 +1,7 @@
 # routers/drafts.py
 from fastapi import APIRouter, HTTPException, BackgroundTasks
 from fastapi import   Depends, Response
+from fastapi.responses import JSONResponse
 import json
 import time
 from percolate.services import MinioService, S3Service
@@ -14,6 +15,10 @@ from percolate.utils import logger
 import traceback
 from percolate.utils.studio import Project, apply_project
 from fastapi import   Depends, File, UploadFile
+import percolate as p8
+from percolate.utils import try_parse_base64_dict
+import time
+
 
 router = APIRouter()
 
@@ -77,14 +82,16 @@ class IndexRequest(BaseModel):
 
  
 @router.post("/index/", response_model=IndexAudit)
-async def index_entity(request: IndexRequest, background_tasks: BackgroundTasks, user: dict = Depends(get_api_key))->IndexAudit:
+async def index_entity(request: IndexRequest, background_tasks: BackgroundTasks, sleep_seconds:int=7, user: dict = Depends(get_api_key))->IndexAudit:
     """index entity and get an audit log id to check status
     the index is created as a background tasks and we respond with an id ref that can be used in the get/
+    we sleep for n seconds to allow records to flush after trigger
     """
     id=uuid.uuid1()
     s = PostgresService(IndexAudit)
+    logger.debug(f"Handling index request {request=} {sleep_seconds=}")
+    time.sleep(sleep_seconds)
     try:
-        
         record = IndexAudit(id=id, model_name='percolate', entity_full_name=request.entity_full_name, metrics={}, status="REQUESTED", message="Indexed requested")
         s.update_records(record)
         """todo create an audit record pending and use that in the api response"""
@@ -113,17 +120,94 @@ async def get_index(id: uuid.UUID, user: dict = Depends(get_api_key)) -> IndexAu
 
 
 @router.post("/content/bookmark")
-async def upload_uri(request : dict, task_id: str = "default", add_resource: bool = True, user: dict = Depends(get_current_token)):
-    """book mark uris the same way we upload file content"""
+async def upload_uri(request : dict,
+                     background_tasks: BackgroundTasks, 
+                     task_id: str = None, 
+                     user_id:str=None,
+                     device_info:str =None, 
+                     is_public_resource:bool = True,
+                     expand_resource:bool=False,
+                     token: dict = Depends(get_current_token)):
+    """book mark uris the same way we upload file content. Resources are assumed public by default e.g. a public we reference but can be "owned" too
+    A task_id is a thread id for a session parent e.g. a chat with actions pinned. but by default we just pin to the daily diary
+    """
 
-    """TODO""" 
+    from percolate.models.p8 import Resources, SessionResources, Session
+    
+    """quickly audit the bookmark and do the rest as a background task
+    im likely to make resources and task resources first class citizens and also create a user session model and functions that do this in the database
+    """
+    
+    """compatibility with modes"""
+    user_id = user_id or request.get('user_id')
+    #
+    """any alias is fine"""
+    uri = request.get('uri') or request.get('url')
+    intent =  request.get('description') or request.get('summary') or request.get('content')
+    resource_title = request.get('title') or request.get('name')
+    """get device info from the base64string"""
+    device_info:dict = try_parse_base64_dict(device_info) or {}
+  
+  
+    if not task_id:
+        task = Session.daily_diary_entry(user_id,query=uri, metadata=device_info)
+        task_id = task.id
+    else:
+        task = Session.task_thread_entry(thread_id=task_id, userid=user_id, query=uri, metadata=device_info)
+    
+    """we are always auditing intent"""
+    p8.repository(Session).update_records(task)   
+        
+    def index_resources():
+        try: 
+      
+            """upsert the resource - allowing for aliases at least for now"""
+             #the intent is the default but the user does not 'own' this public resource and we can process resources in the background - the session stores the user intent
+            head =resources = Resources(name=resource_title,
+                                        content=intent,
+                                        summary = intent, 
+                                        uri = uri,
+                                        userid=None if is_public_resource else user_id)    
+            length = 1
+            if expand_resource:
+                """we attempt to fetch the resource in markdown for the default provider"""
+                resources = Resources.chunked_resource(uri,
+                                                       try_markdown=True,
+                                                       name=resource_title, userid= None if is_public_resource else user_id) 
+                head = resources[0]
+                length = len(resources)
+            
+            """link the resource to sessions
+            We store only the head resource in the session for IIR for now
+            """
+            tr = SessionResources(resource_id=head.id, session_id=task_id,count=length)
+        
+            """for now we insert seps but in future we will have a function for this"""
+            p8.repository(Resources).update_records(resources)
+            p8.repository(SessionResources).update_records(tr)
+            
+            logger.debug(f"Saved task resource {tr=}")
+        except:
+            logger.warning(f"Failing background task")
+            logger.warning(f"{traceback.format_exc()}")
+            """todo implement failure system logs"""
+            
+
+    """background task"""
+    background_tasks.add_task(index_resources)
        
     logger.info(f"{request=}")
        
-    return Response(json.dumps({"status":'received'}))
+    return JSONResponse({"status":f'received uri {uri}'})
 
 @router.post("/content/upload")
-async def upload_file(file: UploadFile = File(...), task_id: str = "default", add_resource: bool = True, user: dict = Depends(get_current_token)):
+async def upload_file(background_tasks: BackgroundTasks,
+                      file: UploadFile = File(...),
+                      task_id: str = None,
+                      add_resource: bool = True, 
+                      user_id:str = None, 
+                      device_info:str =None,
+                      token: dict = Depends(get_current_token)):
     """
     Uploads a file to S3 storage and optionally stores it as a file resource which is indexed.
     Files are stored under the task_id folder structure.
@@ -137,23 +221,62 @@ async def upload_file(file: UploadFile = File(...), task_id: str = "default", ad
     Returns:
         JSON with the filename and status message
     """
- 
+    from percolate.models import Resources,Session,SessionResources
+    device_info = try_parse_base64_dict(device_info)
+                 
+    def index_resource(file_upload_result:dict,task_id:str=None):
+        """given a file upload result which provides e.g. the key, index the resource"""
+        
+        try:         
+            logger.debug(f"indexing {file_upload_result=}")
+            uri = file_upload_result['uri']
+            if not task_id:
+                """the user can upload either in a session context or we just pin to daily activity"""
+                task = Session.daily_diary_entry(userid=user_id,query= uri, metadata=device_info)
+                task_id = task.id
+            else:
+                task = Session.task_thread_entry(thread_id=task_id, userid=user_id, query=uri, metadata=device_info)
+                
+            """we are always auditing intent"""
+            p8.repository(Session).update_records(task)   
+
+            """TODO in future push down S3 urls and add a provider that the chunker and maybe the database can use"""
+            resources = Resources.chunked_resource(name=file.filename, uri=uri,userid=user_id)
+            
+            if resources:
+                _ = p8.repository(Resources).update_records(resources)
+                """resources can be stored as chunked but we store a ref to the head only"""
+                tr = SessionResources(resource_id=resources[0].id, session_id=task_id,count=len(resources))
+                """for now we insert seps but in future we will have a function for this"""
+                p8.repository(SessionResources).update_records(tr)
+                
+                logger.debug(f"uploaded {len(resources)} resource chunks ref={uri}")
+        except:
+            logger.warning(f"Failing background task")
+            logger.warning(f"{traceback.format_exc()}")     
     
     try:
         
         # Upload to S3 using put_object with bytes
         s3_service = S3Service()
+        print("USERID", user_id)
+        path = f'users/{user_id}/' if user_id else ''
+        """todo still deciding on a file path scheme"""
+        path += f"{task_id or 'default'}"
+        
         result = s3_service.upload_file(
-            project_name=task_id,
+            project_name=path,
             file_name=file.filename,
             file_content=file.file,
-            content_type=file.content_type
+            content_type=file.content_type,
+            fetch_presigned_url=True
         )
         
-        # TODO: If add_resource is True, add file metadata to database for indexing
+        if add_resource:
+            background_tasks.add_task(index_resource, file_upload_result=result,task_id=task_id)
         
         logger.info(f"Uploaded file {result['key']} to S3 successfully")
-        return {
+        return JSONResponse({
             "key": result["key"],
             "filename": result["name"],
             "task_id": task_id,
@@ -161,8 +284,9 @@ async def upload_file(file: UploadFile = File(...), task_id: str = "default", ad
             "content_type": result["content_type"],
             "last_modified": result["last_modified"],
             "etag": result["etag"],
+            "path": path,
             "message": "Uploaded successfully to S3"
-        }
+        })
     except Exception as e:
         logger.error(f"File upload failed: {str(e)}")
         logger.warning(traceback.format_exc())
