@@ -7,6 +7,8 @@ from percolate.models.p8 import Function
 from .FunctionManager import FunctionManager
 from percolate.models import AbstractModel, MessageStack
 from percolate.services.llm import CallingContext, FunctionCall, LanguageModel, MessageStackFormatter
+from percolate.services.llm.utils import LLMStreamIterator
+import uuid
 
 GENERIC_P8_PROMPT = """\n# General Advice.
 Use whatever functions are available to you and use world knowledge only if prompted 
@@ -18,6 +20,8 @@ or the user is obviously asking about real world information that is not covered
 - If you encounter a function name that is not your name such as those lists under agent functions, you can activate it with the provided tool to activate_function_by_name assuming you have the correct function name. You should do so without asking the user.
 - When generating large output observe the notification that needs to be called first, call it and continue.
 Of you call a function that is running another agent, you should always pass the full context from the user in the question. Do not shorten or ask simpler questions which loses the users context.
+- you can use the users context if you are given -for example it may be useful to use their recent chat history or ai responses if they seem to be referring to older context.
+ If you do use recent context do not explain or ask for confirmation as this can be jarring for the user.
             """
             
 class ModelRunner:
@@ -190,6 +194,7 @@ class ModelRunner:
         # print(data) # maybe trace here
         """update messages with data if we can or add error messages to notify the language model"""
         self.messages.add(data)
+        return data
 
     @property
     def functions(self) -> typing.Dict[str, Function]:
@@ -210,8 +215,131 @@ class ModelRunner:
         """
         return self.run(question, context, data=data, limit=limit,language_model=language_model)
 
+    def stream(self, question: str, context: CallingContext = None, limit: int = None,
+                   data: typing.List[dict] = None, language_model: str = None, audit: bool = True):
+        """
+        Stream the agentic loop as SSE events.  This method orchestrates successive
+        LLM calls until no function calls remain.  It always yields SSE-formatted
+        text chunks; upon detecting a function call it yields a placeholder event
+        (e.g. "data: im performing function X"), invokes the function, updates the
+        message stack, and continues streaming from the model.
 
-    def run(self, question: str, context: CallingContext = None, limit: int = None, data: typing.List[dict] = None, language_model:str=None):
+        Args:
+            question: The user question to begin the loop.
+            context: Optional CallingContext; if omitted, a new streaming context is used.
+            limit: Max number of agent iterations (function call loops).
+            data: Initial data payload for the agent.
+            language_model: Override the LLM model name.
+            audit: If True, dump audit record after completion.
+
+        Yields:
+            str: SSE-formatted event strings (e.g. "data: ...\n\n").
+        """
+
+        from percolate.services.llm.utils import sse_openai_compatible_stream_with_tool_call_collapse 
+        from percolate.models import AIResponse,User
+        
+        if context:
+            ctx = context.in_streaming_mode(model=language_model)
+        else:
+            ctx = CallingContext.with_model(language_model).in_streaming_mode()
+        self._context = ctx
+        lm_client = LanguageModel.from_context(ctx)
+        # The streaming generator will yield AIResponse objects for auditing directly
+        
+        def _generator():
+            """the generator manages the agentic loop which in turn manage the streaming loops
+            It yields content in the target format i.e. SSE/OpenAI but also sends the internal AIResponse which is not typically sent to the user
+            The AIResponses are turns (tool calls + evals) which are used in the agentic loop but also audited (the main reason we yield them)
+            This process could be simplified if we flush the AIResponse on another thread to not block the user and save them so that we dont need to collect
+            An auditor singleton or repository singleton could just queue up entities and flush them on another thread
+            
+            We write events using a protocol event:
+            """
+            
+            """we can add a users system prompt to the generic and then merge an agents prompt after that"""
+            sys_prompt = GENERIC_P8_PROMPT if not context.plan else  f"{GENERIC_P8_PROMPT}\n\n{context.plan}"
+            # Initialize message stack as in run()
+            payload = data if data is not None else self._init_data
+            self.messages = self.agent_model.build_message_stack(
+                question=question,
+                functions=self.functions.keys(),
+                data=payload,
+                system_prompt_preamble=sys_prompt,
+                user_memory = ctx.get_user_memory()
+            )
+
+            max_loops = limit or ctx.max_iterations
+            last_ai_response = None 
+            saw_stop = False #this is to kill the entire agent loop
+            for _ in range(max_loops):
+                turn_content = ''
+                saw_tool_call = False
+                turn_usage = {}
+                last_ai_response = None
+                raw_response = lm_client._call_raw(
+                    messages=self.messages,
+                    functions=self.function_descriptions,
+                    context=ctx,
+                )
+
+                """if we use non open ai models we have a choice where we want to adapt the deltas TBD
+                in v0 ill probably make the contract openai adapter upstream but for example users may want to relay messages in another scheme
+                there are essential three adaptations we need; function call aggregation needs to be buffered and does not need relay; 
+                token usage also needs to be adapter and does not need relay;
+                the decisions is simply around if the raw content line should be sent in the open ai or other scheme - here its the raw 'line' that is relayed in one scheme or another                
+                """
+                for line, chunk in sse_openai_compatible_stream_with_tool_call_collapse(raw_response):
+                    #print(chunk)
+                    choice = chunk['choices'][0] if chunk.get('choices') else {}
+                    finish = choice.get('finish_reason')
+                    
+                    # Handle tool call batch
+                    if finish == 'tool_calls':
+                        # Tool-call turn: capture usage and mark
+                        saw_tool_call = True
+                        turn_usage = chunk.get('usage', {}) or {}
+                        # Invoke each buffered function call and build aggregate response data
+                        tool_call_evals = {}
+                        for tc in choice['delta'].get('tool_calls', []):
+                            fc = FunctionCall(id=tc['id'], **tc['function'], scheme='openai')
+                            yield f"event: im calling {fc.name}...\n\n"
+                            self.messages.add(fc.to_assistant_message())
+                            tool_call_evals[fc.id] = self.invoke(fc)
+                            
+                        last_ai_response = {
+                            'tool_calls': choice['delta'].get('tool_calls', []),
+                            'tool_eval_data': tool_call_evals,
+                            'function_stack': self.functions.keys()
+                        }
+     
+                    # Stream content deltas
+                    delta = choice.get('delta', {}) or {}
+                    if 'content' in delta:
+                        piece = delta.get('content') or ''
+                        # Send to any streaming callback
+                        if context.streaming_callback:
+                            context.streaming_callback(line)
+                        # Accumulate content for this turn
+                        turn_content += piece
+                        # Yield the SSE-formatted line for client
+                        yield f"{line}\n\n"
+
+                    """in this single request loop"""
+                    if not saw_tool_call:
+                        last_ai_response = {'content': turn_content}
+                    if finish == 'stop':
+                        saw_stop = True
+                    if turn_usage:= chunk.get('usage'):
+                        yield lm_client.parse_ai_response(last_ai_response, turn_usage, ctx)
+                        #saw stop and usage is always last anyway
+                """break out of the agentic loop"""
+                if saw_stop: break
+                     
+        return lm_client.get_stream_iterator(_generator, context=ctx)
+                
+
+    def run(self, question: str, context: CallingContext = None, limit: int = None, data: typing.List[dict] = None, language_model:str=None, audit:bool=True):
         """Ask a question to kick of the agent loop
         
         Args:
@@ -231,8 +359,8 @@ class ModelRunner:
         """messages are system prompt etc. agent model's can override how message stack is constructed
            we add p8 preamble in percolate - in future this could be disabled per model
         """
-        self.messages = self.agent_model.build_message_stack(question=question, functions=self.functions.keys(), data=data,
-                                                             system_prompt_preamble=GENERIC_P8_PROMPT)
+        self.messages = self.agent_model.build_message_stack(question=question, functions=self.functions.keys(),
+                                                             data=data, system_prompt_preamble=GENERIC_P8_PROMPT)
 
         """run the agent loop to completion"""
         for _ in range(limit or self._context.max_iterations):
@@ -252,13 +380,14 @@ class ModelRunner:
                 continue
             if response is not None:
                 # marks the fact that we have unfinished business
-                
                 """for full auditing we should dump the response here"""
                 #p8.repository(AIResponse).update_records(response)
                 break
 
+        
         """fire telemetry TODO and below dump to postgres"""
-        p8.dump(question, self.messages.data, response, self._context, agent=self.agent_model.get_model_full_name())
+        if audit:
+            p8.dump(question, self.messages.data, response, self._context, agent=self.agent_model.get_model_full_name())
 
         return response.content
 
